; _______________________________________________________________________________________
;|                                                                                       |
;| ..:: Fresh IDE ::..  template project.                                                |
;|_______________________________________________________________________________________|
;
;  Description: FreshLib portable console application.
;
;  Target OS: Any, supported by FreshLib
;
;  Dependencies: FreshLib
;
;  Notes:
;_________________________________________________________________________________________

include "%lib%/freshlib.inc"

@BinaryType console, compact

include "%lib%/freshlib.asm"

start:
        InitializeAll

        stdcall GetCmdArguments
        mov     edx, eax

        cmp     [edx+TArray.count], 2
        jb      .finish

        stdcall LoadBinaryFile, [edx+TArray.array+4]
        jc      .finish

        mov     esi, eax
        mov     edi, eax
        mov     dword [esi+ecx], 0

        stdcall FileWriteString, [STDOUT], <txt 'BEGIN;', 13, 10>

.loop_chunk:
        mov     ebx, esi        ; start

.loop_char:
        lodsb
        test    al, al
        jz      .end_of_file

        cmp     al, '\'
        je     .escaped_char

        cmp     al, $0a
        je      .new_line

        cmp     al, $0d
        je      .new_line

        jmp     .loop_char

.new_line:
        push    eax

        lea     ecx, [esi-1]
        sub     ecx, ebx
        jz      .new_line_ok
        stdcall FileWrite, [STDOUT], ebx, ecx
.new_line_ok:
        stdcall FileWrite, [STDOUT], <txt 13, 10>, 2

        pop     eax

        cmp     byte [esi], 0
        je      .final_ok

        xor     al, $0a xor $0d
        cmp     [esi], al
        jne     .loop_chunk
        inc     esi
        jmp     .loop_chunk

.escaped_char:
        lea     ecx, [esi-1]
        sub     ecx, ebx
        jz      .chunk_ok
        stdcall FileWrite, [STDOUT], ebx, ecx
.chunk_ok:
        mov     al, [esi]
        test    al, al
        jz      .end_of_file2

        cmp     al, 'r'
        je      .esc_cr

        cmp     al, 't'
        je      .esc_tab

        cmp     al, '0'
        je      .esc_zero

        cmp     al, 'b'
        je      .esc_bksp

        cmp     al, 'n'
        je      .esc_nl

        cmp     al, 'Z'
        je      .esc_eof

        cmp     byte [esi], "'"
        je      .esc_apos

;simply skip the back slash and output the next char directly.

        stdcall FileWrite, [STDOUT], esi, 1
        inc     esi
        jmp    .loop_chunk

.end_of_file2:
        dec     esi
        stdcall FileWrite, [STDOUT], esi, 1
        jmp     .final_ok

.end_of_file:
        lea     ecx, [esi-1]
        sub     ecx, ebx
        jz      .final_ok
        stdcall FileWrite, [STDOUT], ebx, ecx

.final_ok:
        stdcall FileWriteString, [STDOUT], <txt 13, 10, 'COMMIT;', 13, 10>

        stdcall FreeMem, edi
        jmp     .finish

.esc_zero:
        mov     eax, '@'        ; replace the Zero character!
        jmp     .escapeit

.esc_bksp:
        mov     eax, 8
        jmp     .escapeit

.esc_nl:
        mov     eax, 10
        jmp     .escapeit

.esc_cr:
        mov     eax, 13
        jmp     .escapeit

.esc_tab:
        mov     eax, 9
        jmp     .escapeit

.esc_eof:
        mov     eax, 26
        jmp     .escapeit

.esc_apos:
        mov     eax, "'"
        dec     esi     ; one more time

.escapeit:
        mov     [buffer], eax
        stdcall FileWrite, [STDOUT], buffer, 1
        inc     esi
        jmp     .loop_chunk

.finish:
        FinalizeAll
        stdcall TerminateAll, 0

uglobal
  buffer rd 1
endg